/*
 * Copyright 2008-2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package javarequirementstracer;

import static java.lang.annotation.ElementType.CONSTRUCTOR;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Use this annotation to annotate your code with requirements labels for traceability purposes.
 * <p/>
 * Guidelines:
 * <ul>
 * <li>Typically all exported (ie. public/protected) types (ie. classes/interfaces) in the 
 * presentation and service layer of your system should have this requirements annotation, 
 * linking it to the requirements.
 * Tracing them to lower layer types, general and helper classes, etc. is easy with any IDE.
 * So those lower layer types, etc. don't have to be annotated.
 * <br/>Optionally the model classes in your model subpackage can also be annotated
 * linking them to the requirements domain model.
 * </li>
 * <li>Annotate types specific to a requirement only. Prefer not to annotate general, helper
 * and util classes. For they would require a long unmaintainable list of many requirements. 
 * <br/>NB.1. Helper classes should not be part of a package-API anyway (ie. should not be
 * exposed). {@link JavaRequirementsTracer} does not include package-private types in its 
 * untraceable section. Often the easiest way of removing a type from the untraceable section
 * is by making it package-private (ie. package implementation details should not be exported: 
 * encapsulation)!
 * <br/>NB.2. Exported types that need to be part of the package-API but are not linked 
 * to any requirement can be annotated with
 * {@link SuppressTraceabilityWarnings @SuppressTraceabilityWarnings}. 
 * </li>
 * <li>Prefer annotating a method over a type.</li>
 * <li>All requirements labels known should occur at least once in the main code (100% 
 * requirements coverage). And all exported types should have at least a {@code @Requirements} or
 * {@link SuppressTraceabilityWarnings @SuppressTraceabilityWarnings} annotation (100% code coverage).
 * <br/>NB. This code coverage is not be confused with test coverage which is often also called code coverage. 
 * </li>
 * <li>One could annotate the corresponding unit test classes
 * (see <a href="http://www.junit.org">JUnit</a>) also, but usually that is not of much use.
 * <br/>NB. Unit Tests are white box tests, ie. the test classes reside in the same package as the 
 * tested classes. They can access any member (even private ones if needed) of the tested class. 
 * </li>
 * <li>If system testing is done through code (the prefered way!, eg. using JUnit,
 * <a href="http://jwebunit.sourceforge.net">JWebUnit</a>, etc.), 100% requirements coverage
 * and 100% code coverage should be achieved for the system test code also. 
 * <br/>NB.1. System Tests are black box tests, ie. the test classes reside in <strong>another</strong>
 * package then the tested classes. They access the exported package API only. Since system test code 
 * resides in its own specific packages, one can generate specific reports for main code and system
 * test code coverages, by specifying different packages to scan for requirements labels.
 * <br/>NB.2. Main and System Test code must have separate parameters files but should have a shared labels file.
 * <br/>NB.3. Typically one would write a test class or test method per test scenario. The name of the
 * class/method should reflect the scenario. Multiple test classes/methods (ie. multiple scenarios)
 * can be related to the same set of requirements.
 * </li>
 * </ul>
 * <p/>
 * Reporting (see {@link JavaRequirementsTracer}):
 * <ul>
 * <li>A type is considered annoted with {@code @Requirements} if its type or at least one of 
 * its constructors or methods is annotated with it.
 * </li>
 * <li>Requirements annotations are considered inheritable by subclasses and implementations.
 * {@link JavaRequirementsTracer} considers classes with a supertype annotated with this annotation as traceable,
 * ie. not requiring an explicit annotation of their own. 
 * NB. {@link java.lang.annotation.Inherited java.lang.annotation.Inherited} is not suited for this
 * kind of inheritence, because it works for superclasses only, not for interfaces.
 * </li>
 * </ul>
 * 
 * @author Ronald Koster
 */
@Target({TYPE, METHOD, CONSTRUCTOR})
@Retention(RetentionPolicy.RUNTIME) // NB. Must be RUNTIME instead of SOURCE to facilitate Tracer.
@SuppressTraceabilityWarnings
public @interface Requirements {
	
    /**
     * The set of requirement labels implemented by annotated class, method or constructor.
     * <p/>
     * Requirements labels refer to requirements documents. Usually Use Cases (UCs) or 
     * Supplementary Specifications (SuppSpecs). They can refer to a whole document (doc) 
     * or only a part of it.
     * In the latter case the requirements document itself is usually annotated 
     * (aka. tagged) with detail labels (aka. tags) in the text itself, which can be used 
     * directly as requirements labels. The format of the tags used should be as 
     * agreed with the requirements specifiers.
     * <p/>
     * Examples:
     * <ul>
     * <li>UC01: Refers to UC document of UC 01.</li>
     * <li>UC01.2: Refers to flow 2 of UC document of UC 01.</li>
     * <li>UCD-MAX-AGE: Refers to tag UCD-MAX-AGE in the requirements docs (it is unknown
     * which doc from the tag itself, apparently the tag can easily be found in the requirements
	 * docs).
	 * <br/>NB.1. UCD = Use Case Detail.
	 * <br/>NB.2. It is better to use self-explanatory labels/tags like this one instead of
	 * numbers. From this label one can tell it has something to do with the max age of 
	 * something (what is usually clear from the context). Whereas UCD-137 is totally not 
	 * self-explanatory, it can mean anything.
	 * </li>
     * <li>SUP-ACC-LOCKED: Refers to tag SUP-ACC-LOCKED in the SuppSpecs. It has probably 
     * something to do with accounts being locked or not.
     * </li>
     * </ul> 
     * <p/>
     * WARNING: Do NOT incorporate requirements document version info in the labels (bad practice).
     * Instead, in case different labels are needed to mark different requirements in different
     * versions, use tags (as described above) in the requirements document itself to 
     * uniquely mark those requirements. Also, never reuse a label/tag for another requirement.
     * A new requirement should always get a new label/tag, to ensure the uniqueness 
     * of all labels/tags.
     */
    String[] value();
}
